/********************************/
/* H8/3048FΥץץ */
/* Copyleft,2001,Yukio Mituiwa  */
/********************************/
#include "h8reg.h"
static char hextbl[] = "0123456789ABCDEF";

void putch(unsigned int c)
{
	SCR1 |= 0x20;
	while(!(SSR1 & 0x80));
	TDR1 = c;
	SSR1 &= ~0x80;
	while(!(SSR1 & 0x04));
	SCR1 &= ~0x20;
}

void print(char *str) {
	int i;

	i = 0;
	while(str[i] != 0) putch(str[i++]);
}

void puthexbyte(unsigned char h) {
	putch(hextbl[h >> 4]);
	putch(hextbl[h & 0xf]);
}

void putdecbyte(unsigned char d) {
	int	a, b, f;

	f = 0;
	b = (int)d & 0xff;
	a = (b % 1000) / 100;
	if(a > 0) f = 1;
	if(f == 1) putch(hextbl[a]);
	a = (b % 100) / 10;
	if(a > 0) f = 1;
	if(f == 1) putch(hextbl[a]);
	a = b % 10;
	putch(hextbl[a]);
}

void puthexint(unsigned int h) {
	putch(hextbl[(h >> 28) & 0xf]);
	putch(hextbl[(h >> 24) & 0xf]);
	putch(hextbl[(h >> 20) & 0xf]);
	putch(hextbl[(h >> 16) & 0xf]);
	putch(hextbl[(h >> 12) & 0xf]);
	putch(hextbl[(h >> 8) & 0xf]);
	putch(hextbl[(h >> 4) & 0xf]);
	putch(hextbl[h & 0xf]);
}

void putdecint(unsigned int d) {
	int	a, f;

	f = 0;
	a = (d % 10000000) / 1000000;
	if(a > 0) f = 1;
	if(f == 1) putch(hextbl[a]);
	a = (d % 1000000) / 100000;
	if(a > 0) f = 1;
	if(f == 1) putch(hextbl[a]);
	a = (d % 100000) / 10000;
	if(a > 0) f = 1;
	if(f == 1) putch(hextbl[a]);
	a = (d % 10000) / 1000;
	if(a > 0) f = 1;
	if(f == 1) putch(hextbl[a]);
	a = (d % 1000) / 100;
	if(a > 0) f = 1;
	if(f == 1) putch(hextbl[a]);
	a = (d % 100) / 10;
	if(a > 0) f = 1;
	if(f == 1) putch(hextbl[a]);
	a = d % 10;
	putch(hextbl[a]);
}

int string_check(char *str1, char *str2, int n) {
	int	i, ret;

	ret = 1;
	for(i = 0;i < n;i++) {
		if(str1[i] != str2[i]) {
			ret = 0;
			break;
		}
	}
	return ret;
}

void init() {
	SMR1 = 0;
	BRR1 = 51;
	SCR1 = 0;
	SSR1 = 0;
	P1DDR = 0xff;
	P2DDR = 0xff;
	P5DDR = 0xff;
	P8DDR = 0xff;
}

/* NE2000 쥸ݡ */

#define NE_BASE		0x40000

#define NE_P0_COMMAND	(*(volatile unsigned char *)(NE_BASE))
#define NE_P0_PSTART	(*(volatile unsigned char *)(NE_BASE+1))
#define NE_P0_PSTOP	(*(volatile unsigned char *)(NE_BASE+2))
#define NE_P0_BNRY	(*(volatile unsigned char *)(NE_BASE+3))	/* Boundary Reg */
#define NE_P0_TSR	(*(volatile unsigned char *)(NE_BASE+4))	/* Transmit Status Reg */
#define NE_P0_TPSR	(*(volatile unsigned char *)(NE_BASE+4))	/* Transmit Page Start Address Reg */
#define NE_P0_TBCR0	(*(volatile unsigned char *)(NE_BASE+5))	/* Transmit Byte Counter Reg */
#define NE_P0_NCR	(*(volatile unsigned char *)(NE_BASE+5))	/* Nubmer of Collisions */
#define NE_P0_TBCR1	(*(volatile unsigned char *)(NE_BASE+6))
#define NE_P0_ISR	(*(volatile unsigned char *)(NE_BASE+7))	/* Interrupt Status Reg */
#define NE_P0_RSAR0	(*(volatile unsigned char *)(NE_BASE+8))	/* Remote Start Address Reg */
#define NE_P0_RSAR1	(*(volatile unsigned char *)(NE_BASE+9))
#define NE_P0_RBCR0	(*(volatile unsigned char *)(NE_BASE+10))	/* Remote Byte Count Reg */
#define NE_P0_RBCR1	(*(volatile unsigned char *)(NE_BASE+11))
#define NE_P0_RSR	(*(volatile unsigned char *)(NE_BASE+12))	/* Receive Status Reg */
#define NE_P0_RCR	(*(volatile unsigned char *)(NE_BASE+12))	/* Receive Configuration Reg */
#define NE_P0_TCR	(*(volatile unsigned char *)(NE_BASE+13))	/* Transmit Configuration Reg */
#define NE_P0_DCR	(*(volatile unsigned char *)(NE_BASE+14))	/* Data Configuration Reg */
#define NE_P0_IMR	(*(volatile unsigned char *)(NE_BASE+15))	/* Interrupt Mask Reg */
#define NE_P0_CNTR0	(*(volatile unsigned char *)(NE_BASE+13))
#define NE_P0_CNTR1	(*(volatile unsigned char *)(NE_BASE+14))
#define NE_P0_CNTR2	(*(volatile unsigned char *)(NE_BASE+15))
#define NE_P1_COMMAND	(*(volatile unsigned char *)(NE_BASE))
#define NE_P1_PAR0	(*(volatile unsigned char *)(NE_BASE+1))	/* Physical Addres Reg */
#define NE_P1_PAR1	(*(volatile unsigned char *)(NE_BASE+2))
#define NE_P1_PAR2	(*(volatile unsigned char *)(NE_BASE+3))
#define NE_P1_PAR3	(*(volatile unsigned char *)(NE_BASE+4))
#define NE_P1_PAR4	(*(volatile unsigned char *)(NE_BASE+5))
#define NE_P1_PAR5	(*(volatile unsigned char *)(NE_BASE+6))
#define NE_P1_CURR	(*(volatile unsigned char *)(NE_BASE+7))	/* Current Page Reg */
#define NE_P1_MAR0	(*(volatile unsigned char *)(NE_BASE+8))	/* Multicast Address Reg */

#define NE_ASIC		NE_BASE+16

#define NE_ASIC_DATA	(*(volatile unsigned char *)(NE_ASIC))		/* data port */
#define NE_ASIC_RESET	(*(volatile unsigned char *)(NE_ASIC+15))	/* reset port */

/* NE2000  */
#define	NE_CR_PS0	0x00	/* Page 0 Select   */
#define	NE_CR_PS1	0x40	/* Page 1 Select   */
#define	NE_CR_PS2	0x80	/* Page 2 Select   */
#define	NE_CR_RD0	0x08	/********************** 000:DON'T, 001:read  */
#define	NE_CR_RD1	0x10	/* Remote DMA Control * 010:write, 011:trans */
#define NE_CR_RD2	0x20	/********************** 1xx:disable          */
#define NE_CR_TXP	0x04	/* Transmit packet */
#define NE_CR_STA	0x02	/* start	   */
#define	NE_CR_STP	0x01	/* stop		   */

#define	NE_RCR_SEP	0x01	/* accept error packet */
#define	NE_RCR_AR	0x02	/* accept tiny packet */
#define	NE_RCR_AB	0x04	/* accept broadcast */
#define	NE_RCR_PRO	0x10	/* accept all */
#define	NE_RCR_MON	0x20	/* monitor mode */

#define	NE_DCR_FT0	0x20	/* fifo  */
#define	NE_DCR_FT1	0x40	/* fifo  */
#define	NE_DCR_AR	0x10	/* Auto remote */
#define	NE_DCR_WTS	0x01	/* Word Transfer select */
#define	NE_DCR_BOS	0x02	/* Byte Transfer select */
#define	NE_DCR_LAS	0x04	/* Long Transfer select */
#define	NE_DCR_LS	0x08	/* Loopback select */

#define	NE_ISR_PRX	0x01	/* successful receive */
#define	NE_ISR_PTX	0x02	/* successful trasmit */
#define	NE_ISR_RXE	0x04	/* receive error */
#define	NE_ISR_TXE	0x08	/* transmit error */
#define	NE_ISR_OVW	0x10	/* overflow */
#define	NE_ISR_CNT	0x20	/* counter overflow */
#define	NE_ISR_RDC	0x40	/* Remote DMA complete */
#define	NE_ISR_RST	0x80	/* reset */

#define	NE_IMR_PRXE	0x01	/* successful receive */
#define	NE_IMR_PTXE	0x02	/* successful trasmit */
#define	NE_IMR_RXEE	0x04	/* receive error */
#define	NE_IMR_TXEE	0x08	/* transmit error */
#define	NE_IMR_OVWE	0x10	/* overflow */
#define	NE_IMR_CNTE	0x20	/* counter overflow */
#define	NE_IMR_RDCE	0x40	/* Remote DMA complete */

#define	NE_RSTAT_PRX	0x01	/* successful receive */
#define	NE_RSTAT_CRC	0x02	/* CRC error */
#define	NE_RSTAT_FAE	0x04	/* Frame alignment error */
#define	NE_RSTAT_OVER	0x08	/* overflow */

#define	NE_TSR_PTX	0x01	/* Transmit complete */
#define	NE_TSR_ABT	0x08	/* Transmit Aborted */

#define	NE_TCR_LB0	0x02	/* Internal loopback */
#define	NE_TCR_LB1	0x02	/* Internal loopback */

/* NE2000 ΰ */
#define	NE_MEM_START	 0x40	/* Хåե곫ϥڡֹ	(ɥ쥹4000h) */
#define	NE_MEM_END	 0x80	/* Хåե꽪λڡֹ (ɥ쥹8000h) */
#define	NE_PAGE_SIZE	 256	/* ڡΥХȿ */
#define	NE_TX_PAGE_START 0x40	/* Хåեϥڡֹ */
#define	NE_RX_PAGE_START 0x46	/* Хåեϥڡֹ */
#define	NE_RX_PAGE_STOP  0x80	/* Хåեλڡֹ */

/* ͥå */
#define	ETHER_HEADER_SIZE	14	/* إå */
#define	ETHER_MIN_PACKET	64	/* Ǿѥåȥ */
#define	ETHER_MAX_PACKET	1514	/* ѥåȥ */

/* ե졼ॿ */
#define ARP_TYPE	0x806
#define ARP_REQ		1
#define ARP_ANS		2
#define IP_TYPE		0x800

#define IP_ICMP		1
#define IP_UDP		17
#define IP_TCP		6

char	mac_address[6], ip_address[4];
char	test_message[21] = "NE2000 test message.";
char	test_buffer[21];
char	eeprom[32];
char	ne_cr_default;
char	target_mac[6], target_ipaddr[4], query_ipaddr[4];
char	src_mac[6], dest_mac[6];
char	mac_table[256][6];

void ne_writemem(unsigned char *src, unsigned int dest, unsigned int size) {
	int	i, a;

	/* DMA ߡå׵ǽͭ */
	NE_P0_COMMAND = NE_CR_RD2 | NE_CR_STA;
	
	/* ơ쥸ꥢ */
	NE_P0_ISR = NE_ISR_RDC;

	/* Хȿ */
	NE_P0_RBCR1 = size / 0x100;
	NE_P0_RBCR0 = size % 0x100;

	/* ž襢ɥ쥹 */
	NE_P0_RSAR1 = dest / 0x100;
	NE_P0_RSAR0 = dest % 0x100;

	/* ⡼å׵ǽͭ */
	NE_P0_COMMAND = NE_CR_RD1 | NE_CR_STA;

	/* ⡼ȥХåեإǡž */
	for(i = 0;i < size;i++) NE_ASIC_DATA = src[i];
	
	/* ⡼DMAλޤԤ */
	while(!(NE_P0_ISR & NE_ISR_RDC));
}

void ne_readmem(unsigned int src, unsigned char *dest, unsigned int size) {
	int	i, a;

	/* DMA ߡå׵ǽͭ */
	NE_P0_COMMAND = NE_CR_RD2 | NE_CR_STA;
	
	/* Хȿ */
	NE_P0_RBCR1 = size / 0x100;
	NE_P0_RBCR0 = size % 0x100;

	/* žɥ쥹 */
	NE_P0_RSAR1 = src / 0x100;
	NE_P0_RSAR0 = src % 0x100;

	/* ⡼ȼå׵ǽͭ */
	NE_P0_COMMAND = NE_CR_RD0 | NE_CR_STA;

	/* ⡼ȥХåեǡ */
	for(i = 0;i < size;i++) dest[i] = NE_ASIC_DATA;
}

int ne2000_probe() {
	int i, a, size;
	
	/* եȥꥻå */
	a = NE_ASIC_RESET;
	NE_ASIC_RESET = 0;

	for(a = 0;a < 0x100;a++);	/* Ԥ */

	/* DMA ߡå׵ǽ */
	NE_P0_COMMAND = NE_CR_RD2 | NE_CR_STP;
	for(a = 0;a < 0x100;a++);	/* Ԥ */

	/* ѥåȤ˽񤫤ʤ褦ˤ */
	NE_P0_RCR = NE_RCR_MON;

	/* Хž롼ץХå⡼ */
	NE_P0_DCR = NE_DCR_FT1 | NE_DCR_LS;

	/* Хåեϥɥ쥹 */
	NE_P0_PSTART = NE_MEM_START;

	/* Хåեλɥ쥹 */
	NE_P0_PSTOP = NE_MEM_END;

	/* ХåեRAMΥå */
	ne_writemem(test_message, NE_MEM_START * NE_PAGE_SIZE, 20);
	ne_readmem(NE_MEM_START * NE_PAGE_SIZE, test_buffer, 20);
	test_buffer[20] = 0;
	if(strncmp(test_message, test_buffer, 20) != 0) return -1;

	/* EEPROMɤߤ */
	ne_readmem(0, eeprom, 32);
	for(i = 0;i < 6;i++) mac_address[i] = eeprom[i * 2];

	/* ߥơ쥸ꥢ */
	NE_P0_ISR = 0xff;

	return 0;
}

void ne2000_init() {
	int	i;
	volatile unsigned char *ptr;

	/* ⡼DMA  */
	ne_cr_default = 0;
	NE_P0_COMMAND = ne_cr_default | NE_CR_STP;

	/* FIFO å8Хȡ⡼DMA ưػ */
	/* ȥ륨ǥ8ӥå DMA ž */
	NE_P0_DCR = NE_DCR_FT1 | NE_DCR_LS;

	/* ⡼DMA Хȥ󥿥ꥢ */
	NE_P0_RBCR0 = 0;
	NE_P0_RBCR1 = 0;

	/* ˥⡼(ѥåȼ򤪤ʤХåե󥰤Ϥʤ) */
	NE_P0_RCR = NE_RCR_MON;
	
	/* 롼ץХå⡼ */
	NE_P0_TCR = NE_TCR_LB0;

	/* 󥰥Хåեϥɥ쥹 */
	NE_P0_TPSR = NE_TX_PAGE_START;

	/* 󥰥Хåեϥɥ쥹 */
	NE_P0_PSTART = NE_RX_PAGE_START;

	/* 󥰥Хåեɥ쥹 */
	NE_P0_BNRY = NE_RX_PAGE_START;

	/* 󥰥Хåեλɥ쥹 */
	NE_P0_PSTOP = NE_RX_PAGE_STOP;

	/* ߥơ쥸Υꥢ */
	NE_P0_ISR = 0xff;

	/* ߤȤʤ */
	NE_P0_IMR = 0;

	/* ڡ1 */
	NE_P0_COMMAND = ne_cr_default | NE_CR_PS1 | NE_CR_STP;

	/* Ethernet ɥ쥹(ǻꤷɥ쥹ΥѥåȤ) */
	ptr = &NE_P1_PAR0;
	for(i = 0;i < 6;i++) ptr[i] = mac_address[i];

	/* ǽ˼ѥåȤǼ륢ɥ쥹 */
	NE_P1_CURR = NE_RX_PAGE_START + 1;

	/* ޥ㥹ȥ쥸 */
	ptr = &NE_P1_MAR0;
	for(i = 0;i < 8;i++) ptr[i] = 0;

	/* ڡ0᤹ */
	NE_P0_COMMAND = ne_cr_default | NE_CR_STP;

	/* ѥåȥե륿(֥ɥ㥹ȤȼʬΤߤ˳Ǽ) */
	NE_P0_RCR = NE_RCR_AB;

	/* NIC 򥢥ƥ֤ˤ */
	NE_P0_COMMAND = ne_cr_default | NE_CR_STA;

	/* 롼ץХå⡼ɤȴ̾ư⡼ɤ */
	NE_P0_TCR = 0;
}

int ne2000_output(char *dest_mac, int proto, char *string, int size) {
	int	i, mem_ptr, sendsize, packetsize;
	char	data[8];
	char	buf[1600];

	/* λƤ뤫ɤå */
	while(NE_P0_COMMAND & NE_CR_TXP);

	sendsize = size;
	if(size > (ETHER_MAX_PACKET - ETHER_HEADER_SIZE))
		sendsize = ETHER_MAX_PACKET - ETHER_HEADER_SIZE;
	mem_ptr = NE_TX_PAGE_START * NE_PAGE_SIZE;
	/* MACɥ쥹⡼ȥХåե˽񤭹 */
	ne_writemem(dest_mac, mem_ptr, 6);
	mem_ptr += 6;
	/* MACɥ쥹⡼ȥХåե˽񤭹 */
	ne_writemem(mac_address, mem_ptr, 6);
	mem_ptr += 6;
	/* ץȥ⡼ȥХåե˽񤭹 */
	data[0] = proto / 256;
	data[1] = proto % 256;
	ne_writemem(data, mem_ptr, 2);
	mem_ptr += 2;
	/* ǡ⡼ȥХåե˽񤭹 */
	ne_writemem(string, mem_ptr, sendsize);
	mem_ptr += sendsize;
	/* ѥåȤ礭 */
	packetsize = ETHER_HEADER_SIZE + sendsize;
	if(packetsize < ETHER_MIN_PACKET) packetsize = ETHER_MIN_PACKET;
	/* ޤ򤷤ʤ */
	NE_P0_COMMAND = NE_CR_PS0 | NE_CR_RD2 | NE_CR_STA;
	/* Хåեΰλ */
	NE_P0_TPSR = NE_TX_PAGE_START;
	/* Ĺλ */
	NE_P0_TBCR0 = packetsize % 256;
	NE_P0_TBCR1 = packetsize / 256;
	/* ̿ȯԤ */
	NE_P0_COMMAND = NE_CR_PS0 | NE_CR_TXP | NE_CR_RD2 | NE_CR_STA;

	/* λƤ뤫ɤå */
	while(NE_P0_COMMAND & NE_CR_TXP);

	/* 顼Υå */
	if(NE_P0_TSR & NE_TSR_PTX) return 0;
	else return -1;
}

int ne2000_input(char *string) {
	int	rs_stat, int_stat, bound, current, bound_reg;
	int	ne_rx_start, ne_rx_len, ne_rx_bound;
	int	ne_rx_sub_len, ne_rx_remain_len;
	char	ne_ringbuf_status, ne_ring_buffer[4], *readdata;

	readdata = string;
	/* ڡ0 */
	NE_P0_COMMAND = NE_CR_STA;
	/* ơΥå */
	rs_stat = NE_P0_RSR;
	int_stat = NE_P0_ISR;
	if(rs_stat & NE_RSTAT_OVER) return 0;
	if(int_stat & NE_ISR_OVW) return -1;
	if(!(rs_stat & NE_RSTAT_PRX)) return 0;
	/* ȥڡμ */
	bound = NE_P0_BNRY + 1;
	if(bound >= NE_RX_PAGE_STOP) bound = NE_RX_PAGE_START;
	/* ڡ1 */
	NE_P0_COMMAND = NE_CR_PS1;
	/* ȥڡμ */
	current = NE_P1_CURR;
	if(current >= NE_RX_PAGE_STOP) current = NE_RX_PAGE_START;
	/* ڡ0 */
	NE_P0_COMMAND = NE_CR_PS0;
	/* ǡ̵ͭΥå */
	if(current == bound) return 0;
	/* ڡƬ4ХȤɤ߹ */
	ne_readmem(bound * NE_PAGE_SIZE, ne_ring_buffer, 4);
	ne_ringbuf_status =  ne_ring_buffer[0];
	/* λζ쥸 */
	ne_rx_bound = ne_ring_buffer[1] & 0xff;
	/* ѥåΤγϥɥ쥹 */
	ne_rx_start = bound * NE_PAGE_SIZE + 4;
	/* ѥåΤĹ */
	ne_rx_len = (ne_ring_buffer[3] & 0xff) * 256;
	ne_rx_len |= ne_ring_buffer[2] & 0xff;
	/* ｪλ */
	if( ne_ringbuf_status & NE_RSTAT_PRX &&
	    ne_rx_len >= ETHER_HEADER_SIZE && ne_rx_len <= ETHER_MAX_PACKET ) {
		ne_rx_remain_len = ne_rx_len;
		/* ޤ֤ʬĹ */
		ne_rx_sub_len = NE_RX_PAGE_STOP * NE_PAGE_SIZE - ne_rx_start;
		/* 󥰥Хåեޤ֤뤫 */
		if(ne_rx_sub_len < ne_rx_len) {
			/* ѥåȾɤ߹ */
			ne_readmem(ne_rx_start, readdata, ne_rx_sub_len);
			ne_rx_start = NE_RX_PAGE_START * NE_PAGE_SIZE;
			/* 񤭹ߥɥ쥹ι */
			readdata = &(readdata[ne_rx_sub_len]);
			/* Ĥɤ߹Ĺι */
			ne_rx_remain_len = ne_rx_len - ne_rx_sub_len;
		}
		/* ѥåȤɤ߹ */
		ne_readmem(ne_rx_start, readdata, ne_rx_remain_len);
	} else ne_rx_len = 0;
	bound_reg = (ne_rx_bound == NE_RX_PAGE_START) ? NE_RX_PAGE_STOP : ne_rx_bound;
	bound_reg--;
	NE_P0_BNRY = bound_reg;
	return ne_rx_len;
}

char* make_mac_addr(int a, int b, int c, int d, int e, int f) {
	static	char mac_addr[6];

	mac_addr[0] = a;
	mac_addr[1] = b;
	mac_addr[2] = c;
	mac_addr[3] = d;
	mac_addr[4] = e;
	mac_addr[5] = f;
	return mac_addr;
}

char* make_ip_addr(int a, int b, int c, int d) {
	static char ip_addr[5];

	ip_addr[0] = a;
	ip_addr[1] = b;
	ip_addr[2] = c;
	ip_addr[3] = d;
	return ip_addr;
}

char* make_arp_packet(int type, char *mac, char *ip) {
	static char arp[28];

	arp[0] = 0; arp[1] = 1;
	arp[2] = 8; arp[3] = 0;
	arp[4] = 6; arp[5] = 4;
	arp[6] = 0; arp[7] = type;
	memcpy(arp + 8, mac_address, 6);
	memcpy(arp + 14, ip_address, 4);
	memcpy(arp + 18, mac, 6);
	memcpy(arp + 24, ip, 4);
	return arp;
}

int make_crc(char *buffer, int size) {
	int i, a, b;

	a = 0;
	for(i = 0;i < size;i += 2) {
		b = ((int)buffer[i] & 0xff) * 256;
		b |= (int)buffer[i + 1] & 0xff;
		a = a + b;
		if(a & 0x10000) a++;
		a &= 0xffff;
	}
	return (~a & 0xffff);
}

char* make_ip_header(int proto, int size, char *dest) {
	static char ip[20];
	static int  num = 0;
	int	crc, i;

	num++;
	ip[0] = 0x45; ip[1] = 0;
	ip[2] = (size + 20) / 256; ip[3] = (size + 20) % 256;
	ip[4] = num / 256; ip[5] = num % 256;
	ip[6] = 0; ip[7] = 0;
	ip[8] = 0xff;
	ip[9] = proto;
	ip[10] = 0; ip[11] = 0;
	memcpy(ip + 12, ip_address, 4);
	memcpy(ip + 16, dest, 4);
	crc = make_crc(ip, 20);
	ip[10] = crc / 256; ip[11] = crc % 256;
	return ip;
}

void ip_init() {
	int	i;

	memcpy(ip_address, make_ip_addr(192, 168, 0, 100), 4);
	for(i = 0;i < 256;i++) memcpy(mac_table[i], make_mac_addr(0, 0, 0, 0, 0, 0), 6); 
	memcpy(mac_table[(int)ip_address[3] & 0xff], mac_address, 6);
}

int main(void)
{
	int	crc, i, n, a, frame_type, arptype, ipsize;
	char	*ptr;
	static	char	buffer[ETHER_MAX_PACKET];
	static	char	ipbuf[ETHER_MAX_PACKET];

	init();
	if(ne2000_probe() == -1) {
		print("NE2000 is not found!\n");
		return -1;
	}
	print("Mac address of NE2000 is ");
	for(i = 0;i < 5;i++) {
		puthexbyte(mac_address[i]);
		putch('-');
	}
	puthexbyte(mac_address[i]);
	putch('\n');
	ne2000_init();
	ip_init();
	while(1) {
		n = ne2000_input(buffer);
		if(n > 0) {
			memcpy(dest_mac, buffer, 6);
			memcpy(src_mac, buffer + 6, 6);
			print("\ntarget mac is ");
			for(i = 0;i < 5;i++) {
				puthexbyte(buffer[i]);
				putch('-');
			}
			puthexbyte(buffer[i++]);
			print("\nsource mac is ");
			for(;i < 11;i++) {
				puthexbyte(buffer[i]);
				putch('-');
			}
			puthexbyte(buffer[i++]);
			putch('\n');
			frame_type = (buffer[i] & 0xff) * 256;
			frame_type |= buffer[i + 1] & 0xff;
			if(frame_type == ARP_TYPE) {
				arptype = ((int)buffer[20] & 0xff) << 8;
				arptype += (int)buffer[21] & 0xff;
				if(arptype == 1) {
					memcpy(target_mac, buffer + 22, 6);
					memcpy(target_ipaddr, buffer + 28, 4);
					memcpy(query_ipaddr, buffer + 38, 4);
					if(string_check(ip_address, query_ipaddr, 4)) {
						print("\nmacɥ쥹ϡ");
						for(a = 0;a < 5;a++) {
							puthexbyte(buffer[22 + a]);
							putch('-');
						}
						puthexbyte(buffer[22 + a]);
						print("\nIPɥ쥹ϡ");
						for(a = 0;a < 3;a++) {
							putdecbyte(buffer[28 + a]);
							putch('.');
						}
						puthexbyte(buffer[28 + a]);
						print("\nоIPɥ쥹ϡ");
						for(a = 0;a < 3;a++, i++) {
							putdecbyte(buffer[38 + a]);
							putch('.');
						}
						puthexbyte(buffer[38 + a]);
						putch('\n');
						memcpy(buffer, make_arp_packet(2, target_mac, target_ipaddr), 28);
						ne2000_output(target_mac, 0x806, buffer, 28);
					}
				} else if(arptype == 2) {
					memcpy(mac_table[(int)buffer[31] & 0xff], buffer + 22, 6);
					print("\nmacɥ쥹ϡ");
					for(a = 0;a < 5;a++) {
						puthexbyte(buffer[22 + a]);
						putch('-');
					}
					puthexbyte(buffer[22 + a]);
					print("\nIPɥ쥹ϡ");
					for(a = 0;a < 3;a++) {
						putdecbyte(buffer[28 + a]);
						putch('.');
					}
					puthexbyte(buffer[28 + a]);
					putch('\n');
				}
			} else if(frame_type == IP_TYPE) {
				ipsize = (buffer[16] & 0xff) * 256;
				ipsize |= buffer[17] & 0xff;
				if(buffer[23] == IP_ICMP) {
					memcpy(target_mac, src_mac, 6);
					memcpy(target_ipaddr, buffer + 26, 4);
					memcpy(ipbuf, buffer + 34, ipsize - 20);
					if(buffer[34] == 8) {
						ipbuf[0] = ipbuf[1] = ipbuf[2] = ipbuf[3] = 0;
						crc = make_crc(ipbuf, ipsize - 20);
						ipbuf[2] = crc / 256; ipbuf[3] = crc % 256;
						memcpy(buffer, make_ip_header(IP_ICMP, ipsize - 20, target_ipaddr), 20);
						memcpy(buffer + 20, ipbuf, ipsize - 20);
						ne2000_output(target_mac, 0x800, buffer, ipsize);
					}
				} else if (buffer[23] == IP_UDP){
					print("\nUDPݡȥɥ쥹ϡ");
					a = (buffer[34] & 0xff) * 256;
					a |= buffer[35] & 0xff;
					putdecint(a);
					print("\nUDPݡȥɥ쥹ϡ");
					a = (buffer[36] & 0xff) * 256;
					a |= buffer[37] & 0xff;
					a = a;
					putdecint(a);
					print("\nUDPǡϡ");
					a = (buffer[38] & 0xff) * 256;
					a |= buffer[39] & 0xff;
					a = a - 8;
					putdecint(a);
					putch('\n');
					ptr = &(buffer[42]);
					putch('[');
					for(i = 0;i < a;i++) putch(*(ptr++));
					putch(']');
				} else {
					for(;i < n;i++) {
						putch('(');
						puthexbyte(buffer[i]);
						putch(')');
					}
				}
			} else {
				for(;i < n;i++) {
					putch('[');
					puthexbyte(buffer[i]);
					putch(']');
				}
			}
		}
	}
}
