#include <stdio.h>
#include "../reg3067.h"

#define	RS	0x40
#define	RW	0x80
#define	CS	0x10
#define	D7	0x08
#define	D6	0x04
#define	D5	0x02
#define	D4	0x01
#define	D3	0x08
#define	D2	0x04
#define	D1	0x02
#define	D0	0x01
#define	NONE	0

#define LCDINC		D1
#define LCDSHIFT	D0
#define LCDDISPLAY	D2
#define LCDCURSOR	D1
#define LCDBLINK	D0
#define LCDDISPSHIFT	D3
#define LCDRIGHT	D2
#define LCDLEFT		NONE

static int	current;
static char	dispbuf[20*4];

void init() {
	P1DDR = 0xff;	/* A0-7   is enable */
	P2DDR = 0xff;	/* A8-15  is enable */
	P5DDR = 0x01;	/* A16 is enable */
	P8DDR = 0x0c;	/* CS is enable */
	P4DDR = 0xff;
	P9DDR = 0x30;
	PBDDR = 0xff;
}

void outlcd(int data) {
	static int	i;

	PBDR = CS | data;
	for(i = 0;i < 10;i++);
	PBDR = data;
	for(i = 0;i < 80;i++);	
}

void Clear() {
	static int	i;

	outlcd(NONE); outlcd(D0);
	for(i = 0;i < 3600;i++);	
}

void Home() {
	static int	i;

	outlcd(NONE); outlcd(D1);
	for(i = 0;i < 3600;i++);	
}

void Mode(int com) {
	outlcd(NONE); outlcd(D2 | (com & 0x03));
}

void Control(int com) {
	outlcd(NONE); outlcd(D3 | (com & 0x07));
}

void Cursor(int com) {
	outlcd(D4); outlcd(com & 0x0c);
}

void FuncSet() {
	outlcd(D5); outlcd(D3);
}

void PutChar(int c) {
	static	int	hi, low, i;

	hi = (c >> 4) & 0x0f;
	low = c & 0x0f;
	outlcd(RS | hi);outlcd(RS | low);
	for(i = 0;i < 150;i++);	
}

void Locate(int num) {
	static	int	hi, low, i;

	hi = (num >> 4) & 0x07;
	low = num & 0x0f;
	outlcd(D7 | hi);outlcd(low);
}

void LcdInit() {
	static int	i;

	for(i = 0;i < 100000;i++);
	outlcd(D5 | D4);
	for(i = 0;i < 20000;i++);
	outlcd(D5 | D4);
	for(i = 0;i < 400;i++);	
	outlcd(D5 | D4);
	outlcd(D5);
	FuncSet();
	Control(NONE);
	Clear();
	Mode(LCDINC | LCDSHIFT);
	Control(LCDDISPLAY);
}

void stream_init() {
	static int i;
	
	for(i = 0;i < 80;i++) dispbuf[i] = ' ';
}

void set_locate(int pos) {
	if(pos >= 0 && pos < 20*4)
		current = pos;
}

void put(int c) {
	static int i;
	
	if(c >= 0x20) {
		dispbuf[current++] = c;
	} else if(c == '\n') {
		if(current >= 60) current = 80;
		else if(current >= 40) current = 60;
		else if(current >= 20) current = 40;
		else current = 20;
	}
	if(current >= 80) {
		for(i = 0;i < 60;i++) dispbuf[i] = dispbuf[i + 20];
		for(i = 60;i < 80;i++) dispbuf[i] = ' ';
		current = 60;
	}
}

void flush() {
	static int i;

	Locate(0);
	for(i = 0;i < 20;i++) PutChar(dispbuf[i]);
	for(i = 40;i < 60;i++) PutChar(dispbuf[i]);
	for(i = 20;i < 40;i++) PutChar(dispbuf[i]);
	for(i = 60;i < 80;i++) PutChar(dispbuf[i]);
}

void print(char *p) {
	while(*p != 0) put(*(p++));
	flush();
}

int main(void) {
	init();
	LcdInit();
	stream_init();

	set_locate(0);
	print("H8/3067F\n");
	print("Network controler is programable\n");
	print("TCP/IP protcol\n");
	print("  192.168.0.201");
	while(1);
}
