#include "../reg3664.h"
#define	wait_micro(mi_sec) for(wait = 0;wait < mi_sec;wait++)

void write_pulse(int micro) {
	register int wait;

	TCSRWD = 0x5a;		/* 01011010  ----  TCWD and TMWD is enabled */
	TMWD = 0xff;		/* 11111111  ----  clock x 8192		*/
	TCWD = 255 - 13;	/* 6.6 [mS] 				*/
	TCSRWD = 0xa6;		/* 10100110  ----  WDT is start 	*/
	FLMCR1 |= 0x10;		/* PSU <- 1 				*/
	wait_micro(50);		/* wait 50[uS] 				*/
	FLMCR1 |= 0x01;		/* P <- 1, programing start 		*/
	wait_micro(micro);	/* wait 				*/
	FLMCR1 &= ~0x01;	/* P <- 0, programing stop 		*/
	wait_micro(5);		/* wait 5[uS]				*/
	FLMCR1 &= ~0x10;	/* PSU <- 0 				*/
	wait_micro(5);		/* wait 5[uS] 				*/
	TCSRWD = 0xa2;		/* 10100010  ----  WDT is stop 		*/	
	TCSRWD = 0xaa;		/* 10101010  ----  WDT is disabled 	*/	
}

int write_block(char *dst, char *src) {
	char	rewrite[128], append[128], *verify, compare;
	int	i, n, m;
	register int wait;

	verify = dst;
	FLMCR1 |= 0x40;					/* SWE <- 1 */
	wait_micro(1);					/* wait 1[uS] */
	for(i = 0;i < 128;i++) rewrite[i] = src[i];
	n = 0;
	do {
		n++;
		m = 0;
		for(i = 0;i < 128;i++) dst[i] = rewrite[i]; /* program */
		write_pulse((n <= 6) ? 30: 200);
		FLMCR1 |= 0x04;				/* PV <- 1 */
		wait_micro(4);				/* wait 4[uS] */
		for(i = 0;i < 128;i++) {
			verify[i] = 0xff;		/* dummy write */
			wait_micro(2);			/* wait 2[uS] */
			compare = verify[i];
			if(compare != src[i]) m = 1;	/* compare byte */
			if(n > 6) append[i] = rewrite[i] | verify[i];
			rewrite[i] = src[i] | ~verify[i];
		}
		FLMCR1 &= ~0x04;			/* PV <- 0* /
		wait_micro(2);				/* wait 2[uS] */
		if(n > 6) {
			for(i = 0;i < 128;i++) dst[i] = append[i];
			write_pulse(10);
		}
		if(n > 1000) {
			FLMCR1 &= ~0x40;		/* SWE <- 0 */
			wait_micro(100);		/* wait 100[uS] */
			return -1;			/* ERROR!! */
		}
	} while(m != 0);
	FLMCR1 &= ~0x40;				/* SWE <- 0 */
	wait_micro(100);				/* wait 100[uS] */
	return 0;
}
