#include <stdio.h>
#include "../reg3664.h"
#define CLK_ON	0x80
#define DATA_ON 0x10
#define CLK	0x40
#define DATA	0x20

static char hextbl[] = "0123456789ABCDEF";
void putch(unsigned int c)
{
	while(!(SSR & 0x80));
	TDR = c;
	while(!(SSR & 0x04));
}

void puthexbyte(unsigned char h) {
	putch(hextbl[h >> 4]);
	putch(hextbl[h & 0xf]);
}

void puthexint(unsigned int h) {
	putch(hextbl[(h >> 28) & 0xf]);
	putch(hextbl[(h >> 24) & 0xf]);
	putch(hextbl[(h >> 20) & 0xf]);
	putch(hextbl[(h >> 16) & 0xf]);
	putch(hextbl[(h >> 12) & 0xf]);
	putch(hextbl[(h >> 8) & 0xf]);
	putch(hextbl[(h >> 4) & 0xf]);
	putch(hextbl[h & 0xf]);
}

void init() {
	PMR5 = 0x00;	/* P50-P55 is general I/O */
	PCR5 = 0x00;	/* P50-P57 is input */
	PCR8 = 0xff;	/* P80-P87 is output */
	PCR1 = 0x90;	/* P17,P14 is output */
}

int main(void)
{
	static  unsigned char	data, clock, prev_data, prev_clock;
	static  unsigned char	raw, codes[32];
	static  int	code, count, times, index, i;

	init();
	PDR1 = 0x00;
	data = PDR1 & DATA;
	clock = PDR1 & CLK;
	raw = count = 0;
	times = index = 0;
	while(1) {
		prev_data = data;
		prev_clock = clock;
		data = PDR1 & DATA;
		clock = PDR1 & CLK;
		if(clock != prev_clock && clock == 0) {
			if(count > 0) {
				if(count >= 9 && data != 0) {
					codes[index] = raw;
					if(index < 32) index++;
					raw = count = 0;
				} else {
					raw <<= 1;
					raw += (data == 0) ? 0 : 1;
					count++;
				}
			}
			if(count == 0 && data == 0) {
				times = 0;
				count++;
			}
		}
		times++;
		if(times > 50000) {
			for(i = 0;i < index;i++) {
				putch('[');
				puthexbyte(codes[i]);
				putch(']');
			}
			times = index = 0;
		}
	}
}
