#include <stdio.h>
#include "../reg3664.h"
#define MAX_BUF	80

/* CS of A8-A15 is P74 */
/* P50 -> A8 */ 
/* P57 -> A9 */
/*            */
/* P56 -> A11 */
/* P55 -> A12 */ 
/* P51 -> A13 */ 
/* P54 -> A14 */ 
/* P52 -> A15 */ 
/* P53 -> A16 */ 
/* CS of A16 is P10 */
/* P55 -> A10 */ 

/* P51 -> A7 */
/* P56 -> A6 */
/* P50 -> A5 */
/* P57 -> A4 */
/* P54 -> A3 */
/* P55 -> A2 */
/* P52 -> A1 */
/* P53 -> A0 */

/* P75 -> WE */
/* P76 -> OE */
/* P20 -> CE */

/* P80-P87 -> D0-D7 */

static char hextbl[] = "0123456789ABCDEF";
static char rombuf[128];

void putch(unsigned int c)
{
	while(!(SSR & 0x80));
	TDR = c;
	while(!(SSR & 0x04));
}

void puthexbyte(unsigned char h) {
	putch(hextbl[h >> 4]);
	putch(hextbl[h & 0xf]);
}

void init() {
	PCR1 = 0xff;	/* P17-P14/P10 Ϥˤ  */
	ICCR &= ~0x80;  /* ICE = 0 */
	PCR2 = 0x01;	/* P20 Ϥˤ  */
	PCR5 = 0xff;	/* P50-P57 Ϥˤ  */
	PCR7 = 0xff;	/* P76-P74 Ϥˤ  */
	PCR8 = 0xff;	/* P80-P87 Ϥˤ  */

	PDR1 = 0;
	PDR7 = 0x60;
	PDR2 = 0x01;
}

void addr_set(int addr) {
	PDR5 = 0;
	if(addr & 0x400) PDR5 = 0x20;
	PDR1 |= 0x01;
	PDR1 &= ~0x01;
	PDR5 = 0;
	if(addr & 0x00100) PDR5 |= 0x01;
	if(addr & 0x00200) PDR5 |= 0x80;
	if(addr & 0x00800) PDR5 |= 0x40;
	if(addr & 0x01000) PDR5 |= 0x20;
	if(addr & 0x02000) PDR5 |= 0x02;
	if(addr & 0x04000) PDR5 |= 0x10;
	if(addr & 0x08000) PDR5 |= 0x04;
	if(addr & 0x10000) PDR5 |= 0x08;
	PDR7 |= 0x10;
	PDR7 &= ~0x10;
	PDR5 = 0;
	if(addr & 0x01) PDR5 |= 0x08;
	if(addr & 0x02) PDR5 |= 0x04;
	if(addr & 0x04) PDR5 |= 0x20;
	if(addr & 0x08) PDR5 |= 0x10;
	if(addr & 0x10) PDR5 |= 0x80;
	if(addr & 0x20) PDR5 |= 0x01;
	if(addr & 0x40) PDR5 |= 0x40;
	if(addr & 0x80) PDR5 |= 0x02;	
}

void write_page() {
	static int i;

	PDR2 &= ~0x01;	/* CE = 0 */
	for(i = 0;i < 128;i++) {
		PDR5 &= 0x02;
		if(i & 0x01) PDR5 |= 0x08;
		if(i & 0x02) PDR5 |= 0x04;
		if(i & 0x04) PDR5 |= 0x20;
		if(i & 0x08) PDR5 |= 0x10;
		if(i & 0x10) PDR5 |= 0x80;
		if(i & 0x20) PDR5 |= 0x01;
		if(i & 0x40) PDR5 |= 0x40;
		PDR8 = i;
		PDR7 &= ~0x20;  /* WE = 0 */
		PDR8 = i;
		PDR7 |= 0x20;   /* WE = 1 */
		PDR8 = 0;
	}
	PDR2 |= 0x01;   /* CE = 1 */
	for(i = 0;i < 8000;i++);
}

int read_byte() {
	static int c;
	
	PCR8 = 0;
	PDR2 &= ~0x01;	/* CE = 0 */
	PDR7 &= ~0x40;  /* OE = 0 */
	c = PDR8;
	PDR7 |= 0x40;   /* OE = 1 */
	PDR2 |= 0x01;   /* CE = 1 */
	PCR8 = 0xff;
	return c;
}

int main(void) {
	static int i, c;

	init();
	addr_set(0);
	write_page();
	for(i = 0;i < 128;i++) {
		addr_set(i);
		c = read_byte();
		puthexbyte(c);
	}
}
