#include <stdio.h>
#include "../reg3664.h"
#define MAX_BUF	80

/* CS of A8-A15 is P74 */
/* P50 -> A8 */ 
/* P57 -> A9 */
/*            */
/* P56 -> A11 */
/* P55 -> A12 */ 
/* P51 -> A13 */ 
/* P54 -> A14 */ 
/* P52 -> A15 */ 
/* P53 -> A16 */ 
/* CS of A16 is P10 */
/* P55 -> A10 */ 

/* P51 -> A7 */
/* P56 -> A6 */
/* P50 -> A5 */
/* P57 -> A4 */
/* P54 -> A3 */
/* P55 -> A2 */
/* P52 -> A1 */
/* P53 -> A0 */

/* P75 -> WE */
/* P76 -> OE */
/* P20 -> CE */

/* P80-P87 -> D0-D7 */

static char hextbl[] = "0123456789ABCDEF";
static char rombuf[128];

int getch(void)
{
	unsigned char c, d;

	SCR3 |= 0x10;
	while(!(SSR & 0x40)) {
		if(SSR & 0x38) {
			d = SSR;
			SSR = 0x40;
			return -1;
		}
	}
	c = RDR;
	SCR3 &= ~0x10;
	return c;
}

void putch(unsigned int c)
{
	while(!(SSR & 0x80));
	TDR = c;
	while(!(SSR & 0x04));
}

void print(char *str) {
	int i;

	i = 0;
	while(str[i] != 0) putch(str[i++]);
}

void putstr(char *str) {
	int i;

	i = 0;
	while(str[i] != 0) putch(str[i++]);
	putch(0);
}

void getstr(char *str) {
	int	n,c;

	n = 0;
	c = getch();
	while(c != '\n' && n < MAX_BUF) {
		str[n++] = c;
		c = getch();
	}
	str[n] = 0;
}

int atohex(char *str) {
	int	i, n;
	unsigned int c, val;

	val = i = 0;
	while(str[i] != 0 && i < 8) {
		val <<= 4;
		c = str[i];
		if(c >= '0' && c <= '9') {
			val += c - '0';
		} else if(c >= 'a' && c <= 'f') {
			val += c - 'a' + 10;
		} else if(c >= 'A' && c <= 'F') {
			val += c - 'A' + 10;
		} else {
			return -1;
		}
		i++;
	}
	return val;
}

void puthexbyte(unsigned char h) {
	putch(hextbl[h >> 4]);
	putch(hextbl[h & 0xf]);
}

void puthexint(unsigned int h) {
	putch(hextbl[(h >> 28) & 0xf]);
	putch(hextbl[(h >> 24) & 0xf]);
	putch(hextbl[(h >> 20) & 0xf]);
	putch(hextbl[(h >> 16) & 0xf]);
	putch(hextbl[(h >> 12) & 0xf]);
	putch(hextbl[(h >> 8) & 0xf]);
	putch(hextbl[(h >> 4) & 0xf]);
	putch(hextbl[h & 0xf]);
}


void init() {
	int	i;

	/* SCRν */
	PMR1 = 0x0e;
	SMR = 0;
	BRR = 51;
	SCR3 = 0;
	for(i = 0;i < 200;i++);
	SCR3 = 0x30;
	i = SSR;
	SSR = 0x80;

	/* ݡȤν */
	PCR1 = 0xff;	/* P17-P14/P10 Ϥˤ  */
	ICCR &= ~0x80;  /* ICE = 0 */
	PCR2 = 0x01;	/* P20 Ϥˤ  */
	PCR5 = 0xff;	/* P50-P57 Ϥˤ  */
	PCR7 = 0xff;	/* P76-P74 Ϥˤ  */
	PCR8 = 0xff;	/* P80-P87 Ϥˤ  */

	PDR1 = 0;
	PDR7 = 0x60;
	PDR2 = 0x01;
}

void addr_set(int addr) {
	PDR5 = 0;
	if(addr & 0x400) PDR5 = 0x20;
	PDR1 |= 0x01;
	PDR1 &= ~0x01;
	PDR5 = 0;
	if(addr & 0x00100) PDR5 |= 0x01;
	if(addr & 0x00200) PDR5 |= 0x80;
	if(addr & 0x00800) PDR5 |= 0x40;
	if(addr & 0x01000) PDR5 |= 0x20;
	if(addr & 0x02000) PDR5 |= 0x02;
	if(addr & 0x04000) PDR5 |= 0x10;
	if(addr & 0x08000) PDR5 |= 0x04;
	if(addr & 0x10000) PDR5 |= 0x08;
	PDR7 |= 0x10;
	PDR7 &= ~0x10;
	PDR5 = 0;
	if(addr & 0x01) PDR5 |= 0x08;
	if(addr & 0x02) PDR5 |= 0x04;
	if(addr & 0x04) PDR5 |= 0x20;
	if(addr & 0x08) PDR5 |= 0x10;
	if(addr & 0x10) PDR5 |= 0x80;
	if(addr & 0x20) PDR5 |= 0x01;
	if(addr & 0x40) PDR5 |= 0x40;
	if(addr & 0x80) PDR5 |= 0x02;	
}

void write_page() {
	static int i;

	PDR2 &= ~0x01;	/* CE = 0 */
	for(i = 0;i < 128;i++) {
		PDR5 &= 0x02;
		if(i & 0x01) PDR5 |= 0x08;
		if(i & 0x02) PDR5 |= 0x04;
		if(i & 0x04) PDR5 |= 0x20;
		if(i & 0x08) PDR5 |= 0x10;
		if(i & 0x10) PDR5 |= 0x80;
		if(i & 0x20) PDR5 |= 0x01;
		if(i & 0x40) PDR5 |= 0x40;
		PDR8 = rombuf[i];
		PDR7 &= ~0x20;  /* WE = 0 */
		PDR8 = rombuf[i];
		PDR7 |= 0x20;   /* WE = 1 */
		PDR8 = 0;
	}
	PDR2 |= 0x01;   /* CE = 1 */
	for(i = 0;i < 4000;i++);
}

int read_byte() {
	static int c;
	
	PCR8 = 0;
	PDR2 &= ~0x01;	/* CE = 0 */
	PDR7 &= ~0x40;  /* OE = 0 */
	c = PDR8;
	PDR7 |= 0x40;   /* OE = 1 */
	PDR2 |= 0x01;   /* CE = 1 */
	PCR8 = 0xff;
	return c;
}

int main(void) {
	static int i, data, addr, check;

	init();
	while(1) {
		addr = 0;
		for(i = 0;i < 4;i++) {
			addr <<= 8;
			addr |= getch() & 0xff;
		}
		for(i = 0;i < 128;i++) rombuf[i] = getch();
		addr_set(addr);
		write_page();
		check = 0;
		for(i = 0;i < 128;i++) {
			addr_set(addr + i);
			data = read_byte();
			putch(data);
			if(data != rombuf[i]) check = 1;
		}
		putch((check == 0) ? 'S' : 'F');
	}
}
