/****************************************
 *					*
 * EEPROM write program to H8/3067F	*
 *	Copyleft Yukio Mituiwa,2001	*
 *					*
 ****************************************/

#define	LINUX
#undef	FreeBSD

#include	<stdio.h>
#include	<sys/stat.h>
#include	<unistd.h>
#include	<fcntl.h>
#define	RSLINE	"/dev/ttyS0"
#ifdef LINUX
 #include <termios.h>
 #define sgttyb termios
 #define stty(fd,term) tcsetattr(fd,TCSANOW,term)
#endif
#ifdef FreeBSD
 #include <sgtty.h>
#endif
unsigned char sPort[256], fileName[256], rsbuf[512];
struct sgttyb TheTty;
int TheFd;
FILE *fp;

int putbyte(int c) {
	int	s;

	rsbuf[0] = c;
	if((s = write(TheFd,rsbuf,1)) == -1) {
		fprintf(stderr, "write error\n" );
		exit(-1);
	}
	return s;
}	

int getb() {
	int	s;

	if((s = read(TheFd,rsbuf,1)) == -1) {
		fprintf(stderr, "read error\n" );
		exit(-1);
	}
	return s;
}	

int getbyte() {
	int	s;

	do {
		if((s = read(TheFd,rsbuf,1)) == -1) {
			fprintf(stderr, "read error\n" );
			exit(-1);
		}
	} while(s < 1);
	return rsbuf[0];
}	

int main(argc,argv)
char **argv;
{
	int		i, j, addrsize, romsize, blksize;
	unsigned int	c, n, s, addr, h, bin_size;
	char	buffer[257], numbuf[17], *data, rombuf[0x20000];
	char	enable[4096];
	
	strcpy(sPort,RSLINE);
	switch(argc) {
	case 1:
		fprintf(stderr, "romwrite Motrola-SFile [ serial-port ]\n");
		return -1;
	case 3:
		strcpy(sPort,argv[2]);
	case 2:
		strcpy(fileName,argv[1]);
		break;
	default:
		fprintf(stderr, "option error!!\n");
		return -1;
	}
	if((TheFd=open( sPort , O_RDWR)) == -1){
		fprintf(stderr, "can't open %s\n", sPort);
		return -1;
	}
#ifdef LINUX
	tcsetpgrp(TheFd,getpgrp());
	tcgetattr(TheFd,&TheTty);
	cfsetispeed(&TheTty,9600);
	cfsetospeed(&TheTty,9600);
	TheTty.c_cflag |= HUPCL | CLOCAL ;
	TheTty.c_cflag &= ~( PARENB | CSTOPB );
	cfmakeraw(&TheTty);
#endif
#ifdef FreeBSD
	TheTty.sg_ispeed=B9600;
	TheTty.sg_ospeed=B9600;
	TheTty.sg_flags=RAW;
#endif
	if(stty(TheFd, &TheTty ) == -1){
		fprintf(stderr, "stty error\n" );
		return -1;
	}

	romsize = 0x20000;
	blksize = 4096;
	for(i = 0;i < romsize;i++) rombuf[i] = 0xff;
	for(i = 0;i < blksize;i++) enable[i] = 0;

	if((fp = fopen(fileName,"r")) == NULL) {
		fprintf(stderr,"file not found!\n");
		exit(-1);
	}
	fprintf(stderr, "writing\n");
	fscanf(fp,"%256s",buffer);
	while(!feof(fp)) {
		if(buffer[0] == 'S') {
			if(buffer[1] == '1') {
				addrsize = 1;
				data = buffer + 8;
			} else if(buffer[1] == '2') {
				addrsize = 2;
				data = buffer + 10;

			} else {
/*				fprintf(stderr, "ingore record\n");*/
				fscanf(fp,"%256s",buffer);
				continue;
			}
			for(i = 0;i < 2;i++) numbuf[i] = buffer[i + 2];
			numbuf[i] = '\0';
			sscanf(numbuf,"%X",&s);
			s = s - 2 - addrsize;
			for(i = 0;i < (addrsize + 1) * 2;i++) numbuf[i] = buffer[i + 4];
			numbuf[i] = '\0';
			sscanf(numbuf,"%X",&addr);
			if(addr > romsize) break;
			for(i = 0;i < s;i++) {
				numbuf[0] = *(data++);
				numbuf[1] = *(data++);
				numbuf[2] = '\0';
				sscanf(numbuf,"%X",&h);
				rombuf[addr + i] = h;
				enable[(addr + i)>> 7] = 1;
			}
		}
		fscanf(fp,"%256s",buffer);
	}
	fclose(fp);
	for(i = 0;i < blksize;i++) {
		if(enable[i] == 1) {
			addr = i << 7;
			putbyte((addr >> 24) & 0xff);
			putbyte((addr >> 16) & 0xff);
			putbyte((addr >> 8)& 0xff);
			putbyte(addr & 0xff);
			for(j = 0;j < 128;j++) putbyte(rombuf[addr + j]);
			for(j = 0;j < 128;j++) printf("[%02x]", getbyte() & 0xff);
			putchar(getbyte());
		}
	}
	return 0;
}
