/****************************************
 *					*
 * EEPROM write program to H8/3067F	*
 *	Copyleft Yukio Mituiwa,2001	*
 *					*
 ****************************************/

#define	LINUX
#undef	FreeBSD

#include	<stdio.h>
#include	<sys/stat.h>
#include	<unistd.h>
#include	<fcntl.h>
#define	RSLINE	"/dev/ttyS0"
#ifdef LINUX
 #include <termios.h>
 #define sgttyb termios
 #define stty(fd,term) tcsetattr(fd,TCSANOW,term)
#endif
#ifdef FreeBSD
 #include <sgtty.h>
#endif
#define CPU3048		1
#define CPU3664		2
#define CPU3067		3
unsigned char sPort[256], fileName[256], rsbuf[0x601];
struct sgttyb TheTty;
int TheFd;
FILE *fp;

int putbyte(int c) {
	int	s;

	rsbuf[0] = c;
	if((s = write(TheFd,rsbuf,1)) == -1) {
		fprintf(stderr, "write error\n" );
		exit(-1);
	}
	return s;
}	

int getb() {
	int	s;

	if((s = read(TheFd,rsbuf,1)) == -1) {
		fprintf(stderr, "read error\n" );
		exit(-1);
	}
	return s;
}	

int getbyte() {
	int	s;

	do {
		if((s = read(TheFd,rsbuf,1)) == -1) {
			fprintf(stderr, "read error\n" );
			exit(-1);
		}
	} while(s < 1);
	return rsbuf[0];
}	

void bytecheck(char c,char data) {
	if(c != data) {
		fprintf(stderr, "H8 data %02X ignore!\n",(int)c & 0xff );
		exit(-1);
	}
}

int main(argc,argv)
char **argv;
{
	int		i, j, cpuid, addrsize, romsize, blksize;
	unsigned int	c, n, s, addr, h, bin_size;
	char	buffer[257], numbuf[17], *data, rombuf[0x20000];
	char	binfile[64], enable[4096];
	struct stat fileinfo;
	
	strcpy(sPort,RSLINE);
	switch(argc) {
	case 1:
	case 2:
		fprintf(stderr, "h8write -3664 Motrola-SFile [ serial-port ]\n");
		fprintf(stderr, "h8write -3067 Motrola-SFile [ serial-port ]\n");
		fprintf(stderr, "h8write -3048 Motrola-SFile [ serial-port ]\n");
		return -1;
	case 4:
		strcpy(sPort,argv[3]);
	case 3:
		strcpy(fileName,argv[2]);
		break;
	default:
		fprintf(stderr, "option error!!\n");
		return -1;
	}
	cpuid = 0;
	if(strcmp(argv[1], "-3048") == 0) {
		cpuid = CPU3048;
		strcpy(binfile, "/usr/bin/h8_3048.bin");
	}
	if(strcmp(argv[1], "-3664") == 0) {
		cpuid = CPU3664;
		strcpy(binfile, "/usr/bin/h8_3664.bin");
	}
	if(strcmp(argv[1], "-3067") == 0) {
		cpuid = CPU3067;
		strcpy(binfile, "/usr/bin/h8_3067.bin");
	}
	if(cpuid == 0) {
		fprintf(stderr, "option error!!\n");
		return -1;
	}
	if((TheFd=open( sPort , O_RDWR)) == -1){
		fprintf(stderr, "can't open %s\n", sPort);
		return -1;
	}
#ifdef LINUX
	tcsetpgrp(TheFd,getpgrp());
	tcgetattr(TheFd,&TheTty);
	cfsetispeed(&TheTty,9600);
	cfsetospeed(&TheTty,9600);
	TheTty.c_cflag |= HUPCL | CLOCAL ;
	TheTty.c_cflag &= ~( PARENB | CSTOPB );
	cfmakeraw(&TheTty);
#endif
#ifdef FreeBSD
	TheTty.sg_ispeed=B9600;
	TheTty.sg_ospeed=B9600;
	TheTty.sg_flags=RAW;
#endif
	if(stty(TheFd, &TheTty ) == -1){
		fprintf(stderr, "stty error\n" );
		return -1;
	}

	for(i = 0;i < 256;i++) buffer[i] = 0;
	do {
		if((s = write(TheFd,buffer,256)) == -1) {
			fprintf(stderr, "write error\n" );
			exit(-1);
		}
		s = getb();
	} while(s < 1);
	bytecheck(rsbuf[0], 0);
	putbyte(0x55);
	do {
		s = getbyte();
	} while(s < 1);
	bytecheck(rsbuf[0], 0xaa);
	switch(cpuid) {
	case CPU3067:
		romsize = 0x20000;
		blksize = 4096;
		fprintf(stderr,"H8/3067F is ready!  2001/2/1 Yukio Mituiwa.\n");
		break;
	case CPU3048:
		romsize = 0x20000;
		blksize = 4096;
		fprintf(stderr,"H8/3048F is ready!  2001/2/1 Yukio Mituiwa.\n");
		break;
	case CPU3664:
		romsize = 0x8000;
		blksize = 256;
		fprintf(stderr,"H8/3664F is ready!  2001/2/1 Yukio Mituiwa.\n");
		break;
	}
	putbyte(0x06); getbyte();
	putbyte(0x00); getbyte();
 	if((fp = fopen(binfile,"r")) == NULL) {
		fprintf(stderr, binfile);
		fprintf(stderr, " is not installed!\n" );
		exit(-1);
	}
	fread(rsbuf,0x600,1,fp);
	write(TheFd,rsbuf,0x600);
	for(i = 0;i < 0x600;i++) getbyte();
	c = getbyte();
	fclose(fp);
	if(c == 0xaa) {
		puts("EEPROM writing program transfer successed.");
	}
	else {
		puts("EEPROM writing program transfer failed!");
		exit(-1);
	}
	if((fp = fopen(fileName,"r")) == NULL) {
		fprintf(stderr,"file not found!\n");
		exit(-1);
	}
	fprintf(stderr, "writing\n");
	for(i = 0;i < romsize;i++) rombuf[i] = 0xff;
	for(i = 0;i < blksize;i++) enable[i] = 0;
	fscanf(fp,"%256s",buffer);
	while(!feof(fp)) {
		if(buffer[0] == 'S') {
			if(buffer[1] == '1') {
				addrsize = 1;
				data = buffer + 8;
			} else if(buffer[1] == '2') {
				addrsize = 2;
				data = buffer + 10;

			} else {
				fprintf(stderr, "ingore record\n");
				fscanf(fp,"%256s",buffer);
				continue;
			}
			for(i = 0;i < 2;i++) numbuf[i] = buffer[i + 2];
			numbuf[i] = '\0';
			sscanf(numbuf,"%X",&s);
			s = s - 2 - addrsize;
			for(i = 0;i < (addrsize + 1) * 2;i++) numbuf[i] = buffer[i + 4];
			numbuf[i] = '\0';
			sscanf(numbuf,"%X",&addr);
			if(addr > romsize) break;
			for(i = 0;i < s;i++) {
				numbuf[0] = *(data++);
				numbuf[1] = *(data++);
				numbuf[2] = '\0';
				sscanf(numbuf,"%X",&h);
				rombuf[addr + i] = h;
				switch(cpuid) {
				case CPU3664:
					enable[(addr + i)>> 7] = 1;
					break;
				case CPU3048:
				case CPU3067:
					enable[(addr + i)>> 5] = 1;
					break;
				}
			}
		}
		fscanf(fp,"%256s",buffer);
	}
	fclose(fp);
	for(i = 0;i < blksize;i++) {
		if(enable[i] == 1) {
			switch(cpuid) {
			case CPU3067:
				putbyte((i >> 8) & 0xff);
				putbyte(i & 0xff);
				for(j = 0;j < 32;j++) putbyte(rombuf[i * 32 + j]);
				for(j = 0;j < 32;j++) {
					if(j % 16 == 0) fprintf(stderr, "\n%04x : ", i * 32 + j);
					fprintf(stderr, "%02x ", getbyte() & 0xff);
				}
				break;
			case CPU3048:
				putbyte((i >> 11) & 0xff);
				putbyte((i >> 3) & 0xff);
				putbyte((i << 5) & 0xff);
				putbyte(32);
				for(j = 0;j < 32;j++) putbyte(rombuf[i * 32 + j]);
				for(j = 0;j < 32;j++) {
					if(j % 16 == 0) fprintf(stderr, "\n%04x : ", i * 32 + j);
					fprintf(stderr, "%02x ",rombuf[i * 32 + j]  & 0xff);
				}
				break;
			case CPU3664:
				putbyte(i);
				for(j = i * 128;j < i * 128 + 128;j++) putbyte(rombuf[j]);
				for(j = 0;j < 128;j++) {
					if(j % 16 == 0) fprintf(stderr, "\n%04x : ", i * 128 + j);
					fprintf(stderr, "%02x ", getbyte() & 0xff);
				}
				break;
			}
			c = getbyte();
			if(c != 'S') {
				fprintf(stderr,"\nEEPROM Writing is failed.\n");
				return -1;
			}
		}
	}
	fprintf(stderr,"\nEEPROM Writing is successed.\n");
	return 0;
}
