/********************************/
/* H8/3048FΥץץ */
/* Copyleft,2001,Yukio Mituiwa  */
/********************************/
#include "h8reg.h"
static char hextbl[] = "0123456789ABCDEF";

void putch(unsigned int c)
{
	SCR1 |= 0x20;
	while(!(SSR1 & 0x80));
	TDR1 = c;
	SSR1 &= ~0x80;
	while(!(SSR1 & 0x04));
	SCR1 &= ~0x20;
}

void print(char *str) {
	int i;

	i = 0;
	while(str[i] != 0) putch(str[i++]);
}

void puthexbyte(unsigned char h) {
	putch(hextbl[h >> 4]);
	putch(hextbl[h & 0xf]);
}

void puthexint(unsigned int h) {
	putch(hextbl[(h >> 28) & 0xf]);
	putch(hextbl[(h >> 24) & 0xf]);
	putch(hextbl[(h >> 20) & 0xf]);
	putch(hextbl[(h >> 16) & 0xf]);
	putch(hextbl[(h >> 12) & 0xf]);
	putch(hextbl[(h >> 8) & 0xf]);
	putch(hextbl[(h >> 4) & 0xf]);
	putch(hextbl[h & 0xf]);
}

void init() {
	SMR1 = 0;
	BRR1 = 51;
	SCR1 = 0;
	SSR1 = 0;
	P8DDR = 0x0e;	/* xxxx1110 */
}

#define DO	1
#define DI	2
#define CLK	4
#define CS	8
#define wait() {int i;for(i=0;i<3;i++);}
#define wait_tcs() {int i;P8DR=0;for(i=0;i<8;i++);}
#define WEN	0x130
#define WDS	0x100
#define READ	0x180
#define WRITE	0x140
#define WRALL	0x100

int command1(int com) {
	int	mask, port;	

	P8DR = CS;
	wait();
	for(mask = 0x100;mask != 0;mask >>= 1) {
		port = ((com & mask) ? DI : 0) | CS;
		P8DR = port;
		wait();
		P8DR = port | CLK;
		wait();
	}
}

void write(int addr, int data) {
	int	i, mask, port;
	
	command1(WRITE | (addr & 0x3f));
	for(mask = 0x8000;mask != 0;mask >>= 1) {
		port = ((data & mask) ? DI : 0) | CS;
		P8DR = port;
		wait();
		P8DR = port | CLK;
		wait();
	}
	wait_tcs();
	P8DR = CS;
	while(!(P8DR & DO));
	P8DR = 0;
}

void write_all(int data) {
	int	i, mask, port;
	
	command1(WRALL);
	for(mask = 0x8000;mask != 0;mask >>= 1) {
		port = ((data & mask) ? DI : 0) | CS;
		P8DR = port;
		wait();
		P8DR = port | CLK;
		wait();
	}
	wait_tcs();
	P8DR = CS;
	while(!(P8DR & DO));
	P8DR = 0;
}

int read(int addr) {
	int	i, data;
	
	data = 0;
	command1(READ | (addr & 0x3f));
	for(i = 0;i <= 16;i++) {
		P8DR = CS;
		if(P8DR & DO) data++;
		data <<= 1;
		wait();
		P8DR = CS | CLK;
		wait();
	}
	wait_tcs();
	return data >> 1;
}

int main(void)
{
	int	i, data;

	init();
	P8DR = 0;
	wait();
	command1(WEN); wait_tcs();
	write(0, 0x40);
	write(1, 0x10);
	write(2, 0xee00);
	write(3, 0x00ee);
	write(4, 0x0100);
	write(5, 0);
	write(6, 0);
	write(7, 0);
	write(8, 0);
	command1(WDS); wait_tcs();
	for(i = 0;i < 10;i++) {
		data = read(i);
		puthexbyte(i); putch(':');
		puthexint(data); putch('\n');
	}
}
