/********************************/
/* H8/3664FΥץץ */
/* Copyleft,1999,Yukio Mituiwa  */
/********************************/
#include <stdio.h>
#include "../reg3664.h"
void out7seg(int, int, int);
#define SW1DATA	0x08
#define SW2DATA	0x04
#define SW3DATA	0x02
#define SW4DATA	0x01
#define SW1MASK	0x88
#define SW2MASK	0x44
#define SW3MASK	0x22
#define SW4MASK	0x11
static unsigned int sw_data, dispmode, mode, timermode;
static unsigned int sec, min, hour;
static unsigned int set_hour, set_min, set_sec, limitcount;

void init() {
	mode = dispmode = timermode = 0;
	sec = min = hour = limitcount = 0;
	PMR1 = 0x0f;	/* P14-P17 is I/O, TxD RxD is enable */
	PCR1 = 0x00;	/* P10-P17 is output */
	PUCR1 = 0x08;	/* P10 -P17 not pull-up */
	ICCR = 0x00;	/* IIC is disable */
	PMR5 = 0x00;	/* P50-P55 is I/O */
	PCR5 = 0xc3;	/* P50,P51,P56.P57 is output, P52-P55 is input */
	PUCR5 = 0x3c;	/* P52-P55 is pull-up */
	PCR7 = 0x70;	/* P74-P76 is output */	
	PCR8 = 0xff;	/* P80-P87 is output */
/*	SYSCR1 = 0x88;
	SYSCR2 = 0x00;
	MSTCR1 = 0x7e;*/
}

int main(void) {
	init();
	TMA = 0x1a;	/* 1 / 4 [sec] */
	IENR1 = 0x40;
	asm("andc.b\t#127,ccr\n");
	while(1) {
		sw_data |= ( ~PDR5 >> 2 ) & 0x0f;
	}
}

void out7seg(int dec2, int dec1, int dot) {
	int	i, seg[4];

	if(dec1 == -1 || dec2 == -1) {
		PDR7 = 0x20;
		PDR8 = 0x04;
		return;
	}
	seg[3] = dec1 / 10;
	seg[2] = dec1 % 10;
	seg[1] = dec2 / 10;
	seg[0] = dec2 % 10;
	PDR7 = 0;
	PDR8 = 0;
	if(dot == 0) PDR8 |= 0x40;
	if(dot == 1) PDR8 |= 0x20;
	if(dot == 2) PDR8 |= 0x10;
	if(dot == 3) PDR8 |= 0x80;
	for(i = 0;i < 4;i++) {
		PDR7 &= ~0x30;
		PDR8 &= ~0x0c;
		if(seg[i] & 8) PDR7 |= 0x20;
		if(seg[i] & 4) PDR8 |= 0x04;
		if(seg[i] & 2) PDR8 |= 0x08;
		if(seg[i] & 1) PDR7 |= 0x10;
		if(i == 0) PDR8 |= 0x01; 
		if(i == 1) PDR7 |= 0x40; 
		if(i == 2) PDR8 |= 0x02; 
	}
}

void one_sec() {
	static char lamp[4] = {0x80, 0xc0, 0x40, 0};
	
	if(sec >= 59) {
		sec = 0;
		if(min >= 59) {
			min = 0;
			if (hour >= 23) hour = 0;
			else hour++;
		} else {
			min++;
		}
	} else {
		sec++;
	}
	switch(mode) {
	case 0:
		if(dispmode == 0) out7seg(min, sec, 2);
		else if(dispmode == 1) out7seg(hour, min, 2);
		if(sec < 6) PDR5 |= 0x02;
		else PDR5 &= ~0x02;
		break;
	case 2:
		if(timermode == 1) {
			if(set_sec >= 59) {
				set_sec = 0;
				if(set_min >= 59) set_min = 0;
				else set_min++;
			} else set_sec++;
			out7seg(set_min, set_sec, 2);
			if((limitcount / 2) == set_min && ((limitcount % 2) * 30) == set_sec) PDR5 |= 0x02;
		}
		break;
	}
	PDR5 = (PDR5 & 0x03) | lamp[mode];
}

void keyboard(int key) {
	switch(key) {
	case SW1DATA:
		switch(mode) {
		case 0:
			dispmode = (dispmode == 0) ? 1 : 0;
			if(dispmode == 0) out7seg(min, sec, 2);
			else if(dispmode == 1) out7seg(hour, min, 2);
			break;
		case 1:
			if(set_min == -1 || set_hour == -1) {
				set_min = min;
				set_hour = hour;
			}
			if(set_min >= 59) set_min = 0;
			else set_min++;
			out7seg(set_hour, set_min, 2);
			break;
		case 2:
			timermode = (timermode == 0) ? 1 : 0;
			break;
		}
		break;
	case SW2DATA:
		switch(mode) {
		case 1:
			if(set_min == -1 || set_hour == -1) {
				set_min = min;
				set_hour = hour;
			}
			if(set_min >= 49) set_min = set_min % 10;
			else set_min += 10;
			out7seg(set_hour, set_min, 2);
			break;
		case 2:
			if(timermode == 0) {
				set_min = set_sec = 0;
				out7seg(set_min, set_sec, 2);
			}
			if(timermode == 1) PDR5 &= ~0x02;
			break;
		}
		break;
	case SW3DATA:
		switch(mode) {
		case 1:
			if(set_min == -1 || set_hour == -1) {
				set_min = min;
				set_hour = hour;
			}
			if(set_hour >= 23) set_hour = 0;
			else set_hour++;
			out7seg(set_hour, set_min, 2);
			break;
		case 2:
			if(limitcount >= 19) limitcount = 0;
			else limitcount++;
			out7seg(limitcount / 2, (limitcount % 2) * 30, 2);
			break;
		}
		break;
	case SW4DATA:
		if(mode >= 3) mode = 0;
		else mode++;
		PDR5 &= ~0x03;
		switch(mode) {
		case 1:
			set_hour = set_min = -1;
			out7seg(hour, min, 2);
			break;
		case 2:
			if(set_min != -1 && set_hour != -1) {
				hour = set_hour;
				min = set_min;
				sec = 0;
			}
			timermode = 0;
			set_min = set_sec = 0;
			out7seg(set_min, set_sec, 2);
			break;
		case 3:
			out7seg(-1, -1, -1);
			PDR5 &= ~0x03;
			break;
		}
		break;
	}
}

#pragma interrupt
void int_timera() {
	static int count = 0, swcount[4] = {0,0,0,0};
	int i, data, mask;

	for(i = 0, data = 0x01, mask = 0x11;i < 4;i++, data <<= 1, mask <<= 1) {
		if((sw_data & mask) == data && swcount[i] > 2) {
			keyboard(data);
			swcount[i] = 0;
		}
		swcount[i]++;
	}
	if(count >= 3) {
		count = 0;
		one_sec();
	} else count++;
	sw_data = (sw_data << 4) & 0xf0;
	IRR1 &= ~0x40;
}
